@@ -1,9 +1,9 @@
-use crate::fixer::Fixer;
+use crate::fixer::{Fixer, FixerError};
 use crate::rule::referent_rule::RuleRegistration;
 use crate::rule::Rule;
 use crate::rule_config::RuleConfigError;
 use crate::rule_core::RuleCoreError;
-use crate::transform::{TransformError, Transformation};
+use crate::transform::{Transform, TransformError};
 use crate::RuleCore;
 
 use std::collections::{HashMap, HashSet};
@@ -22,8 +22,8 @@ pub fn check_rule_with_hint<'r>(
   rule: &'r Rule,
   utils: &'r RuleRegistration,
   constraints: &'r HashMap<String, Rule>,
-  transform: &'r Option<HashMap<String, Transformation>>,
-  fixer: &Option<Fixer>,
+  transform: &'r Option<Transform>,
+  fixer: &Vec<Fixer>,
   hint: CheckHint<'r>,
 ) -> RResult<()> {
   match hint {
@@ -37,6 +37,9 @@ pub fn check_rule_with_hint<'r>(
     }
     // upper_vars is needed to check metavar defined in containing vars
     CheckHint::Rewriter(upper_vars) => {
+      if fixer.is_empty() {
+        return Err(RuleCoreError::Fixer(FixerError::InvalidRewriter));
+      }
       check_utils_defined(rule, constraints)?;
       check_vars_in_rewriter(rule, utils, constraints, transform, fixer, upper_vars)?;
     }
@@ -48,8 +51,8 @@ fn check_vars_in_rewriter<'r>(
   rule: &'r Rule,
   utils: &'r RuleRegistration,
   constraints: &'r HashMap<String, Rule>,
-  transform: &'r Option<HashMap<String, Transformation>>,
-  fixer: &Option<Fixer>,
+  transform: &'r Option<Transform>,
+  fixer: &Vec<Fixer>,
   upper_var: &HashSet<&str>,
 ) -> RResult<()> {
   let vars = get_vars_from_rules(rule, utils);
@@ -74,8 +77,8 @@ fn check_vars<'r>(
   rule: &'r Rule,
   utils: &'r RuleRegistration,
   constraints: &'r HashMap<String, Rule>,
-  transform: &'r Option<HashMap<String, Transformation>>,
-  fixer: &Option<Fixer>,
+  transform: &'r Option<Transform>,
+  fixer: &Vec<Fixer>,
 ) -> RResult<()> {
   let vars = get_vars_from_rules(rule, utils);
   let vars = check_var_in_constraints(vars, constraints)?;
@@ -115,7 +118,7 @@ fn check_var_in_constraints<'r>(
 
 fn check_var_in_transform<'r>(
   mut vars: HashSet<&'r str>,
-  transform: &'r Option<HashMap<String, Transformation>>,
+  transform: &'r Option<Transform>,
 ) -> RResult<HashSet<&'r str>> {
   let Some(transform) = transform else {
     return Ok(vars);
@@ -140,13 +143,12 @@ fn check_var_in_transform<'r>(
   Ok(vars)
 }
 
-fn check_var_in_fix(vars: HashSet<&str>, fixer: &Option<Fixer>) -> RResult<()> {
-  let Some(fixer) = fixer else {
-    return Ok(());
-  };
-  for var in fixer.used_vars() {
-    if !vars.contains(&var) {
-      return Err(RuleCoreError::UndefinedMetaVar(var.to_string(), "fix"));
+fn check_var_in_fix(vars: HashSet<&str>, fixers: &Vec<Fixer>) -> RResult<()> {
+  for fixer in fixers {
+    for var in fixer.used_vars() {
+      if !vars.contains(&var) {
+        return Err(RuleCoreError::UndefinedMetaVar(var.to_string(), "fix"));
+      }
     }
   }
   Ok(())
